
import sys
import itertools
from factcheck import *

# Pretty rudimentary tests.  In particular, they test nothing about distributions.

@forall(x=always("foo"))
def test_always_always_returns_same_value(x):
    assert x == "foo"

@forall(x=[1,2], y=[20,30])
def test_can_specify_fixed_sequence_of_inputs(x, y):
    assert x in (1,2) and y in (20,30)

@forall(x=[1,2], samples=300)
def test_can_specify_fixed_sequence_of_inputs_which_is_then_repeated(x):
    assert x == 1 or x == 2

class TestCanApplyForallDecoratorToTestMethods:
    @forall(x=ints())
    def test_example_method(self, x):
        type(x) == int

@forall(v=choices([3, 6, 2, 1, 11]))
def test_choice_selects_random_elements_from_a_sequence(v):
    assert v in [3, 6, 2, 1, 11]

@forall(n=ints(-3,4))
def test_ints_generates_ints_between_min_and_max_inclusive(n):
    assert -3 <= n <= 4

@forall(n=ints())
def test_ints_generates_ints_between_default_min_and_max_inclusive(n):
    assert default_min_int <= n <= default_max_int

@forall(n=from_range(start=2, stop=22, step=4))
def test_from_range_generates_ints_between_start_inclusive_stop_exclusive_by_step(n):
    assert 2 <= n < 22
    assert n%4 == 2

@forall(l=lists(lengths=ints(min=2,max=4), elements=ints(min=10,max=20)))
def test_lists_generates_random_length_lists_with_random_elements(l):
    assert 2 <= len(l) <= 4
    assert all(10 <= e <= 20 for e in l)

@forall(l=lists())
def test_lists_with_no_parameters_generates_lists_of_default_lengths_with_integer_elements(l):
    assert 0 <= len(l) <= 32
    assert all(type(i) == int for i in l)

@forall(t=tuples(ints(2,4), ints(3,9)))
def test_tuples_generates_random_fixed_size_tuples(t):
    assert 2 <= t[0] <= 4
    assert 3 <= t[1] <= 9

@forall(d=dicts({'a':ints(3,7), 'b':ints(0,6)}))
def test_dicts_generates_dicts_with_fixed_keys_and_random_values(d):
    assert 3 <= d['a'] <= 7
    assert 0 <= d['b'] <= 6

@forall(i=bitseqs(lengths=ints(min=3,max=4)))
def test_bitseqs_generates_integers_with_random_bits_of_random_size(i):
    assert 0 <= i < 16

@forall(f=floats(1.0, 3.0))
def test_floats_generates_floating_point_numbers_within_range(f):
    assert 1.0 <= f < 3.0

@forall(f=floats())
def test_floats_generates_floating_point_numbers_within_default_range(f):
    assert default_min_float <= f <= default_max_float

def _f(a, b, c, d):
    return (a+b-c)+d

@forall(n=mapping(_f, always(1), b=always(2), d=always(3), c=always(4)))
def test_mapping_generates_result_of_mapping_callable_over_argument_generators(n):
    assert n == 2

def test_unique_returns_unique_elements_in_order():
    assert list(unique([1,2,3,2,1])) == [1,2,3]
    assert list(unique([])) == []

def test_unique_identifier_can_be_defined_by_key_function():
    assert list(unique([-1,-2,-3,1,2,3,4], key=abs)) == [-1, -2, -3, 4]


@forall(x=range(4), y=range(4), where=lambda x, y: x != y)
def test_can_filter_inputs(x, y):
    assert x != y

@forall(x=range(4), y=range(4), z=range(4), where=lambda x, y: x != y)
def test_do_not_need_to_refer_to_all_parameters_in_filter(x, y, z):
    assert x != y




import sys
import itertools
from factcheck import *

@forall
def test_can_specify_inputs_as_python_3_parameter_attributes(x:always("foo")):
    assert x == "foo"

@forall(samples=300)
def test_can_configure_test_samples_when_using_python_3_parameter_attributes(x:always("foo")):
    assert x == "foo"



#!/usr/bin/env python

from setuptools import setup
from setuptools.command.test import test as TestCommand
import sys
import os

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_suite = True
        self.test_args = ['test/test_factcheck.py']
        if sys.version_info[0] > 2:
            self.test_args.append('test/test_factcheck_python3.py')
        self.test_args.append('--duration=10')
        
    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


setup(name='factcheck',
      version='1.2.0.0',
      description='A simple, extensible implementation of QuickCheck for Python',
      author='Nat Pryce',
      author_email='about-factcheck@natpryce.com',
      url='http://github.com/npryce/python-factcheck',
      
      license="Apache 2.0",
      
      classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Natural Language :: English',
        'Topic :: Software Development :: Testing',
      ],
      
      provides=['factcheck'],
      py_modules=['factcheck'],
      
      tests_require=['pytest'],
      cmdclass = {'test': PyTest}
)



"""A simple but extensible implementation of QuickCheck 1 for Python 3.

Copyright (c) 2012 Nat Pryce.
"""

import sys
import random
from itertools import product, cycle, repeat, islice
import inspect


if sys.version_info[0] > 2:
    imap = map
    izip = zip
else:
    from itertools import imap, izip


def _random_values(_generator_fn, *args, **kwargs):
    while True:
        yield _generator_fn(*args, **kwargs)

def _defaulted(value, default_value):
    return default_value if value is None else value



def always(v):
    """always returns 'v'"""
    return repeat(v)


def choices(seq):
    """random element from non-empty sequence 'seq'"""
    return _random_values(random.choice, seq)


def bitseqs(lengths):
    """long ints with 'n' random bits"""
    return (random.getrandbits(length) for length in lengths)


default_min_int = -1000
"""Default min value for ints"""

default_max_int = +1000
"""Default max value for ints"""

def ints(min=None, max=None):
    """random integers between min and max, inclusive"""
    return _random_values(random.randint, 
                          _defaulted(min, default_min_int), 
                          _defaulted(max, default_max_int))


def from_range(start, stop=None, step=1):
    """random integers taken from range(start, stop[, step])"""
    return _random_values(random.randrange, start, stop, step)


default_min_float = -1000
"""Default min value for floats"""

default_max_float = 1000
"""Default max value for floats"""

def floats(lower=None, upper=None):
    """random floating-point numbers selected uniformly from [a,b) or [a,b] depending on rounding."""
    return _random_values(random.uniform,
                          _defaulted(lower, default_min_float),
                          _defaulted(upper, default_max_float))

default_sequence_lengths = ints(min=0, max=32)
"""Default lengths for sequences and lists"""

default_sequence_elements = ints()
"""Default elements for sequences and lists"""

def sequences(lengths=None, elements=None):
    """random length sequences of random elements"""
    elements = _defaulted(elements, default_sequence_elements)
    lengths = _defaulted(lengths, default_sequence_lengths)
    
    return (islice(elements, length) for length in lengths)


def lists(lengths=None, elements=None):
    """random length lists of random elements"""
    return imap(list, sequences(lengths, elements))


def tuples(*elementses):
    """fixed-size tuples of random elements."""
    return izip(*elementses)


def dicts(d):
    """dicts with fixed keys and random values.
    
    Parameters:
       d : a dictionary mapping keys to generators
      
    Returns: a generator of dictionaries, mapping each key, k, of d to
       the next element of d[k].
    """
    keys, value_iters = izip(*d.items())
    return (dict(izip(keys,values)) for values in izip(*value_iters))


def mapping(f, *args_gens, **kwargs_gens):
    return (f(*args,**kwargs) for (args, kwargs) in izip(tuples(*args_gens), dicts(kwargs_gens)))


def unique(elements, key=(lambda x:x)):
    """Yield unique elements, preserving order."""
    seen = set()
    for element in elements:
        k = key(element)
        if k not in seen:
            seen.add(k)
            yield element


def _annotations(f):
    return f.__annotations__ if hasattr(f, "__annotations__") else {}

def _always(*args, **kwargs):
    return True

def _params(param_bindings, f):
    argspec = inspect.getargspec(f)
    if argspec.keywords is not None:
        return param_bindings
    else:
        return dict((k,param_bindings[k]) for k in argspec.args)


def forall(_test_fn=None, samples=1000, where=_always, **parameter_generators):    
    where_argspec = inspect.getargspec(where)
    if where_argspec.keywords is None:
        where_bindings = lambda d: {k:d[k] for k in where_argspec.args}
    else:
        where_bindings = lambda d: d
    
    def bind_parameters(test_fn):
        parameter_generators.update(_annotations(test_fn))
        
        # Note: should be decorated by @functools.wraps(test_fn) but that confuses pytest
        def bound_test_fn(*args):
            param_names, param_value_iters = zip(*parameter_generators.items())
            param_value_samples = islice(zip(*[cycle(i) for i in param_value_iters]), 0, samples)
            
            for param_values in param_value_samples:
                param_bindings = dict(zip(param_names, param_values))
                
                if where(**where_bindings(param_bindings)):
                    test_fn(*args, **param_bindings)
        
        return bound_test_fn
    
    # Allow @forall, @forall(samples=100) or @forall(param1=generator1, param2=generator2, ...)
    return bind_parameters if _test_fn is None else bind_parameters(_test_fn)

